package ru.ulstu.tsMethods;

import ru.ulstu.models.Forecast;
import ru.ulstu.models.ModelTimeSeries;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.TimeSeriesValidateException;

import java.util.HashMap;
import java.util.Map;

public abstract class TimeSeriesMethod {
    protected TimeSeries originalTimeSeries;
    protected ModelTimeSeries model;
    protected Forecast forecast;
    protected int countForecast;
    protected Map<Param, Double> parameters = new HashMap<>();

    protected TimeSeriesMethod(TimeSeries originalTimeSeries) throws TimeSeriesValidateException {
        validateTimeSeries(originalTimeSeries);
        this.originalTimeSeries = originalTimeSeries;
        model = new ModelTimeSeries(originalTimeSeries);
        forecast = new Forecast(originalTimeSeries);
    }

    public TimeSeries getOriginalTimeSeries() {
        return originalTimeSeries;
    }

    public ModelTimeSeries getModel() {
        return model;
    }

    public Forecast getForecast() {
        return forecast;
    }

    public abstract void createModel();

    private void validateTimeSeries(TimeSeries timeSeries) throws TimeSeriesValidateException {
        if (timeSeries == null || timeSeries.isEmpty()) {
            throw new TimeSeriesValidateException("Временной ряд должен быть не пустым");
        }
        if (timeSeries.getValues().stream().anyMatch(val -> val == null || val.getValue() == null)) {
            throw new TimeSeriesValidateException("Временной ряд содержит пустые значения");
        }
        if (timeSeries.getValues().stream().anyMatch(val -> val.getDate() == null)) {
            throw new TimeSeriesValidateException("Временной ряд должен иметь отметки времени");
        }
    }
}
